import java.lang.*;
import java.util.*;
import org.w3c.dom.*;

public class Pattern{

    Snode patnode;         
    int rmb;               
    ArrayList nodelist;    
    int rootocc;           
    ArrayList negnodelist; 
    int negrootocc;        

    Pattern(String nodename){
	patnode = new Snode(nodename);
	rmb = 0;
	nodelist = new ArrayList();
	negnodelist = new ArrayList();
	rootocc = 0;
	negrootocc = 0;
    }

    Pattern(Snode node,int p){
	patnode = node;
	rmb = p;
	nodelist = new ArrayList();
	negnodelist = new ArrayList();
	rootocc = 0;
	negrootocc = 0;
    }

    public Snode getPatNode(){
        return patnode;
    }

    public int getrmb(){
        return rmb;
    }

    public ArrayList getNodeList(){
        return nodelist;
    }

    public ArrayList getNegNodeList(){
	return negnodelist;
    }

    public int getRootOcc(){
	return rootocc;
    }

    public int getRMOcc(){
	return nodelist.size();
    }

    public int getNegRootOcc(){
	return negrootocc;
    }

    public int getNegRMOcc(){
	return negnodelist.size();
    }


    public void setRootOcc(int i){
	rootocc = i;
    }

    public void setNegRootOcc(int i){
	negrootocc = i;
    }


    public void addNode(Node node){
	nodelist.add(node);
    }

    public void addNegNode(Node node){
	negnodelist.add(node);
    }


    public Node getNode(int i){
	return (Node)nodelist.get(i);
    }

    public Node getNegNode(int i){
	return (Node)negnodelist.get(i);
    }
}


class Snode{

    String nodename;     
    Snode firstchild;    
    Snode nextsibling;   
    Snode parent;        

    Snode(String s){
        nodename = new String(s);
        firstchild = null;
        nextsibling = null;
        parent = null;
    }

    Snode(Snode p, String s){
        nodename = new String(s);
        firstchild = null;
        nextsibling = null;
        parent = p;
    }

    public String getNodeName(){
        return nodename;
    }

    public Snode getFirstChild(){
        return firstchild;
    }

    public Snode getNextSibling(){
        return nextsibling;
    }

    public Snode getParent(){
        return parent;
    }

    public Snode getRootNode(){
        Snode target = this;

        while (target.parent != null)
            target = target.parent;

        return target;
    }

    public Snode getRML(){
        Snode target = getRootNode();

        while (target.firstchild != null){
            target = target.firstchild;
            while (target.nextsibling != null)
                target = target.nextsibling;
        }

        return target;
    }

    private Snode addChild(String nodename){

        Snode target;
        Snode child = new Snode(this,nodename);

        if (firstchild == null){
            firstchild = child;
        }
        else{
            target = firstchild;
            while(target.nextsibling != null)
                target = target.nextsibling;
            target.nextsibling = child;
        }

        return child;
    }

    public Snode expand(int p,String l){

        Snode target;

        if (p == 0){
            target = addChild(l);
        }
        else{
            target = parent;
            p--;

            while (p > 0){
                target = target.parent;
                p--;
            }

            target = target.addChild(l);
        }
        return target;
    }

    public Snode copypattern(){

        Snode rootnode = getRootNode();
        String nodename = rootnode.getNodeName();
        Snode current = new Snode(nodename);
        Snode nextnode = rootnode.getFirstChild();

        while (nextnode != null){
            current = current.copypatternsub(nextnode);
            nextnode = nextnode.getNextSibling();
            current = current.getParent();
        }
        current = current.getRML();
        return current;
    }

    private Snode copypatternsub(Snode node){

        String nodename = node.getNodeName();
        Snode current = addChild(nodename);
        Snode nextnode = node.getFirstChild();

        while (nextnode != null){
            current = current.copypatternsub(nextnode);
            nextnode = nextnode.getNextSibling();
            current = current.getParent();
        }
        return current;
    }

    public void print(){

	Snode node = this.getRootNode();
	System.out.print("(");
	printsub(node);
	System.out.print(")");
    }

    private void printsub(Snode node){

        String nodename = node.getNodeName();
        System.out.print(nodename);
        Snode nextnode = node.getFirstChild();

        while (nextnode != null){
            System.out.print("(");
            printsub(nextnode);
            System.out.print(")");
            nextnode = nextnode.getNextSibling();
        }
    }

    public String printTagStyle(){

	Snode node = this.getRootNode();
	return printTSSub(node);
    }

    private String printTSSub(Snode node){

	String str;

	if (node.nodename.equals("#text") == true)
	    str = "#text ";
	else if (node.nodename.equals("#comment") == true)
	    str = "#comment ";
	else if (node.nodename.startsWith("[") &&
		 node.nodename.endsWith("]"))
	    str = node.nodename.substring(1, node.nodename.length() - 1);
	else if (node.nodename.startsWith("@") == true)
	    str = node.printAttributeEdge();
	else{
	    boolean tagclose = false;
	    str = "<" + node.nodename;
	    for (Snode target = node.firstchild; target != null;
		 target = target.nextsibling){
		if (target.nodename.startsWith("@") == true)
		    str = str + " " + target.printAttributeEdge();
		else{
		    if (tagclose == false){
			str = str + "> " + printTSSub(target);
			tagclose = true;
		    }
		    else
			str = str + printTSSub(target);
		}
	    }
	    if (tagclose == false)
		str = str + " /> ";
	    else
		str = str + " </" + node.nodename + "> ";
	}

	return str;
    }

    private String printAttributeEdge(){

	String str;
	if (this.firstchild != null){
	    String attrValue = this.firstchild.nodename;
	    str = this.nodename.substring(1) + "=\""
		+ attrValue.substring(1, attrValue.length()-1) + "\"";
	}
	else
	    str = this.nodename.substring(1);
	return str;
    }
}





